use std::net::ToSocketAddrs;

use clap::Parser;
use futures::StreamExt;

#[derive(clap::Parser)]
#[command(author, version, about, long_about = None)]
struct Opts {
    #[command(subcommand)]
    subcommand: Option<Subcommand>,
}

#[derive(clap::Subcommand)]
enum Subcommand {
    Serve {
        #[clap(long, short = 'l', default_value = "127.0.0.1", env)]
        listen_address: String,
        #[clap(long, short = 'p', default_value = "3000", env)]
        listen_port: u16,
    },

    #[cfg(feature = "extract-static")]
    Extract { destination: String },
}

impl Default for Subcommand {
    fn default() -> Self {
        Subcommand::Serve {
            listen_address: "127.0.0.1".into(),
            listen_port: 3000,
        }
    }
}

#[tokio::main]
async fn main() -> Result<(), Box<dyn std::error::Error>> {
    let opts = Opts::parse();

    tracing_subscriber::fmt::init();

    match opts.subcommand.unwrap_or_default() {
        Subcommand::Serve {
            listen_address,
            listen_port,
        } => {
            let addrs: Vec<_> = (listen_address.as_str(), listen_port)
                .to_socket_addrs()?
                .collect();

            let service = chat_signaling_server::web::routes::make_service()?;

            let mut futures =
                futures::stream::FuturesUnordered::from_iter(addrs.iter().map(|addr| async {
                    let addr = addr.clone();
                    tracing::info! {?addr, "app is running..."};

                    axum::Server::bind(&addr).serve(service.clone()).await
                }));
            while let Some(Err(error)) = futures.next().await {
                tracing::error!(?error, "failed");
                unreachable!()
            }
        }

        #[cfg(feature = "extract-static")]
        Subcommand::Extract { destination } => {
            staticfiles::extract(destination).await?;
        }
    }

    Ok(())
}
